*******************************************************************************************************
* Program name: DANE_2_QUAIDS.do
* Program purpose: This estimates the QUAIDS model and derives price and income elasticities for the Czech Republic
* Program created:   June 2011 - October 2013, by Petr Jansky as a part of the TACR OMEGA project and in cooperation with the IFS
*******************************************************************************************************

* Basic set-up of the QUAIDS model - set the number of the group, number of groups, number of the generate dataset used, the time period of the data used, number of bootstrapping loops
clear all
global group = 1 // 5/9: 1 2 3 4 5 6 7 8 (1 4 6 8),  7/10: 9 10 11 12 13 14 15 21, 8: 1 8 22 23 24 25 26 27, 11: 1997-2010 ** Choose the groups of expenditures you want (change n to be a number of the group that you want)
global numgoods = 8
global numdata = 11
global startyear = 2001
global endyear = 2011
global numbootstrap = 1300 // number of bootstraps

* Type of elasticities computation
		global elast "2" // I had to copy "2" directly on a row around 580 for this to work - do not know why
		* 1: Elasticities for one representative household. the elasticities at mean prices and total expenditure, and for a representative household with two adults and two children living in Prague where the head-of-household is a male and has a middle level of education and is employed
		* 2: Elasticities for expenditure-weighted average households - This one is used/presented. These elasticities are calculated for each household individually, and then a weighted average is constructed, with the weights being equal to the household's share of total sample expenditure for the relevant good 
* Set up for the estimates of VAT simulations	
	* choose your favourite elasticity - hicksian_3_ or marshall_3_
	global elast "marshall_3_" 
	
** Set up file directories and some globals
global folder "C:\Users\PJ\Desktop\IDEA\1107_Demand_system\Data"
*global folder "D:\LTaxes\QUAIDS\Data"
cd "$folder\outputs"
global sru "$folder\SRU_PJ"
global outputs "$folder\outputs"
global data "$folder\"

* What variables should be kept
* For analysis of excise duties
global excise_duties "sj exp3900 exp3901 exp3902 exp3903 exp3640 exp2830 exp2840 exp2841 exp2842 exp2850 exp2930 exp2931 exp2932 exp2940 exp2941 exp2942 exp2950 exp2951 exp2952 amount2830 amount2840 amount2841 amount2842 amount2850 amount2930 amount2931 amount2932 amount2940 amount2941 amount2942 amount2950 amount2951 amount2952"
* For changes in VAT groups
global VAT_selected_groups "amount2010-exp5350" // all expenditure and amount variables in the sru_all_11 file


* Demographic variables to be included in the QUAIDS model (and an extended version with the residual variables v)
global odems "age osob child empstat educlow educmid praha city_size rok_year" // 
global dems "age osob child empstat educlow educmid v1 v2 v3 v4 v5 v6 praha city_size rok_year"   // or only cubic terms - v3 and v6 to be included? (TBD)  

* QUAIDS parameters and elasticities results - variable to present the results
global results "b1_1	b1_2	b1_3	b1_4	b1_5	b1_6	b1_7	b1_8	c1	d1	w1_age	w1_osob	w1_child	w1_empstat	w1_educlow	w1_educmid	w1_v1	w1_v2	w1_v3	w1_v4	w1_v5	w1_v6	w1_praha	w1_city_size	w1_rok_year	a1	b2_1	b2_2	b2_3	b2_4	b2_5	b2_6	b2_7	b2_8	c2	d2	w2_age	w2_osob	w2_child	w2_empstat	w2_educlow	w2_educmid	w2_v1	w2_v2	w2_v3	w2_v4	w2_v5	w2_v6	w2_praha	w2_city_size	w2_rok_year	a2	b3_1	b3_2	b3_3	b3_4	b3_5	b3_6	b3_7	b3_8	c3	d3	w3_age	w3_osob	w3_child	w3_empstat	w3_educlow	w3_educmid	w3_v1	w3_v2	w3_v3	w3_v4	w3_v5	w3_v6	w3_praha	w3_city_size	w3_rok_year	a3	b4_1	b4_2	b4_3	b4_4	b4_5	b4_6	b4_7	b4_8	c4	d4	w4_age	w4_osob	w4_child	w4_empstat	w4_educlow	w4_educmid	w4_v1	w4_v2	w4_v3	w4_v4	w4_v5	w4_v6	w4_praha	w4_city_size	w4_rok_year	a4	b5_1	b5_2	b5_3	b5_4	b5_5	b5_6	b5_7	b5_8	c5	d5	w5_age	w5_osob	w5_child	w5_empstat	w5_educlow	w5_educmid	w5_v1	w5_v2	w5_v3	w5_v4	w5_v5	w5_v6	w5_praha	w5_city_size	w5_rok_year	a5	b6_1	b6_2	b6_3	b6_4	b6_5	b6_6	b6_7	b6_8	c6	d6	w6_age	w6_osob	w6_child	w6_empstat	w6_educlow	w6_educmid	w6_v1	w6_v2	w6_v3	w6_v4	w6_v5	w6_v6	w6_praha	w6_city_size	w6_rok_year	a6	b7_1	b7_2	b7_3	b7_4	b7_5	b7_6	b7_7	b7_8	c7	d7	w7_age	w7_osob	w7_child	w7_empstat	w7_educlow	w7_educmid	w7_v1	w7_v2	w7_v3	w7_v4	w7_v5	w7_v6	w7_praha	w7_city_size	w7_rok_year	a7	b8_1	b8_2	b8_3	b8_4	b8_5	b8_6	b8_7	a8	c8	d8	b8_8	w8_age	w8_osob	w8_child	w8_empstat	w8_educlow	w8_educmid	w8_v1	w8_v2	w8_v3	w8_v4	w8_v5	w8_v6	w8_praha	w8_city_size	w8_rok_year IncElast_1 IncElast_2 IncElast_3 IncElast_4 IncElast_5 IncElast_6 IncElast_7 IncElast_8 hicksian1_1 hicksian2_1 hicksian3_1	hicksian4_1	hicksian5_1	hicksian6_1	hicksian7_1	hicksian8_1 hicksian1_2	hicksian2_2	hicksian3_2	hicksian4_2	hicksian5_2	hicksian6_2	hicksian7_2	hicksian8_2 hicksian1_3	hicksian2_3	hicksian3_3	hicksian4_3	hicksian5_3	hicksian6_3	hicksian7_3	hicksian8_3 hicksian1_4	hicksian2_4	hicksian3_4	hicksian4_4	hicksian5_4	hicksian6_4	hicksian7_4	hicksian8_4 hicksian1_5	hicksian2_5	hicksian3_5	hicksian4_5	hicksian5_5	hicksian6_5	hicksian7_5	hicksian8_5 hicksian1_6	hicksian2_6	hicksian3_6	hicksian4_6	hicksian5_6	hicksian6_6	hicksian7_6	hicksian8_6 hicksian1_7	hicksian2_7	hicksian3_7	hicksian4_7	hicksian5_7	hicksian6_7	hicksian7_7	hicksian8_7 hicksian1_8	hicksian2_8	hicksian3_8	hicksian4_8	hicksian5_8	hicksian6_8	hicksian7_8	hicksian8_8 marshall1_1	marshall2_1	marshall3_1	marshall4_1	marshall5_1	marshall6_1	marshall7_1	marshall8_1 marshall1_2	marshall2_2	marshall3_2	marshall4_2	marshall5_2	marshall6_2	marshall7_2	marshall8_2 marshall1_3	marshall2_3	marshall3_3	marshall4_3	marshall5_3	marshall6_3	marshall7_3	marshall8_3 marshall1_4	marshall2_4	marshall3_4	marshall4_4	marshall5_4	marshall6_4	marshall7_4	marshall8_4 marshall1_5	marshall2_5	marshall3_5	marshall4_5	marshall5_5	marshall6_5	marshall7_5	marshall8_5 marshall1_6	marshall2_6	marshall3_6	marshall4_6	marshall5_6	marshall6_6	marshall7_6	marshall8_6 marshall1_7	marshall2_7	marshall3_7	marshall4_7	marshall5_7	marshall6_7	marshall7_7	marshall8_7 marshall1_8	marshall2_8	marshall3_8	marshall4_8	marshall5_8	marshall6_8	marshall7_8	marshall8_8 symmetry1_1	symmetry2_1	symmetry3_1	symmetry4_1	symmetry5_1	symmetry6_1	symmetry7_1	symmetry8_1 symmetry1_2	symmetry2_2	symmetry3_2	symmetry4_2	symmetry5_2	symmetry6_2	symmetry7_2	symmetry8_2 symmetry1_3	symmetry2_3	symmetry3_3	symmetry4_3	symmetry5_3	symmetry6_3	symmetry7_3	symmetry8_3 symmetry1_4	symmetry2_4	symmetry3_4	symmetry4_4	symmetry5_4	symmetry6_4	symmetry7_4	symmetry8_4 symmetry1_5	symmetry2_5	symmetry3_5	symmetry4_5	symmetry5_5	symmetry6_5	symmetry7_5	symmetry8_5 symmetry1_6	symmetry2_6	symmetry3_6	symmetry4_6	symmetry5_6	symmetry6_6	symmetry7_6	symmetry8_6 symmetry1_7	symmetry2_7	symmetry3_7	symmetry4_7	symmetry5_7	symmetry6_7	symmetry7_7	symmetry8_7 symmetry1_8	symmetry2_8	symmetry3_8	symmetry4_8	symmetry5_8	symmetry6_8	symmetry7_8	symmetry8_8 zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year kraj skup osob2 age2 sex educ pmes child2 pkoef"
global results3 "b1_1	b1_2	b1_3	b1_4	b1_5	b1_6	b1_7	b1_8	c1	d1	w1_age	w1_osob	w1_child	w1_empstat	w1_educlow	w1_educmid	w1_v1	w1_v2	w1_v3	w1_v4	w1_v5	w1_v6	w1_praha	w1_city_size	w1_rok_year	a1	b2_1	b2_2	b2_3	b2_4	b2_5	b2_6	b2_7	b2_8	c2	d2	w2_age	w2_osob	w2_child	w2_empstat	w2_educlow	w2_educmid	w2_v1	w2_v2	w2_v3	w2_v4	w2_v5	w2_v6	w2_praha	w2_city_size	w2_rok_year	a2	b3_1	b3_2	b3_3	b3_4	b3_5	b3_6	b3_7	b3_8	c3	d3	w3_age	w3_osob	w3_child	w3_empstat	w3_educlow	w3_educmid	w3_v1	w3_v2	w3_v3	w3_v4	w3_v5	w3_v6	w3_praha	w3_city_size	w3_rok_year	a3	b4_1	b4_2	b4_3	b4_4	b4_5	b4_6	b4_7	b4_8	c4	d4	w4_age	w4_osob	w4_child	w4_empstat	w4_educlow	w4_educmid	w4_v1	w4_v2	w4_v3	w4_v4	w4_v5	w4_v6	w4_praha	w4_city_size	w4_rok_year	a4	b5_1	b5_2	b5_3	b5_4	b5_5	b5_6	b5_7	b5_8	c5	d5	w5_age	w5_osob	w5_child	w5_empstat	w5_educlow	w5_educmid	w5_v1	w5_v2	w5_v3	w5_v4	w5_v5	w5_v6	w5_praha	w5_city_size	w5_rok_year	a5	b6_1	b6_2	b6_3	b6_4	b6_5	b6_6	b6_7	b6_8	c6	d6	w6_age	w6_osob	w6_child	w6_empstat	w6_educlow	w6_educmid	w6_v1	w6_v2	w6_v3	w6_v4	w6_v5	w6_v6	w6_praha	w6_city_size	w6_rok_year	a6	b7_1	b7_2	b7_3	b7_4	b7_5	b7_6	b7_7	b7_8	c7	d7	w7_age	w7_osob	w7_child	w7_empstat	w7_educlow	w7_educmid	w7_v1	w7_v2	w7_v3	w7_v4	w7_v5	w7_v6	w7_praha	w7_city_size	w7_rok_year	a7	b8_1	b8_2	b8_3	b8_4	b8_5	b8_6	b8_7	a8	c8	d8	b8_8	w8_age	w8_osob	w8_child	w8_empstat	w8_educlow	w8_educmid	w8_v1	w8_v2	w8_v3	w8_v4	w8_v5	w8_v6	w8_praha	w8_city_size	w8_rok_year IncElast_1 IncElast_2 IncElast_3 IncElast_4 IncElast_5 IncElast_6 IncElast_7 IncElast_8 hicksian1_1 hicksian2_1 hicksian3_1	hicksian4_1	hicksian5_1	hicksian6_1	hicksian7_1	hicksian8_1 hicksian1_2	hicksian2_2	hicksian3_2	hicksian4_2	hicksian5_2	hicksian6_2	hicksian7_2	hicksian8_2 hicksian1_3	hicksian2_3	hicksian3_3	hicksian4_3	hicksian5_3	hicksian6_3	hicksian7_3	hicksian8_3 hicksian1_4	hicksian2_4	hicksian3_4	hicksian4_4	hicksian5_4	hicksian6_4	hicksian7_4	hicksian8_4 hicksian1_5	hicksian2_5	hicksian3_5	hicksian4_5	hicksian5_5	hicksian6_5	hicksian7_5	hicksian8_5 hicksian1_6	hicksian2_6	hicksian3_6	hicksian4_6	hicksian5_6	hicksian6_6	hicksian7_6	hicksian8_6 hicksian1_7	hicksian2_7	hicksian3_7	hicksian4_7	hicksian5_7	hicksian6_7	hicksian7_7	hicksian8_7 hicksian1_8	hicksian2_8	hicksian3_8	hicksian4_8	hicksian5_8	hicksian6_8	hicksian7_8	hicksian8_8 marshall1_1	marshall2_1	marshall3_1	marshall4_1	marshall5_1	marshall6_1	marshall7_1	marshall8_1 marshall1_2	marshall2_2	marshall3_2	marshall4_2	marshall5_2	marshall6_2	marshall7_2	marshall8_2 marshall1_3	marshall2_3	marshall3_3	marshall4_3	marshall5_3	marshall6_3	marshall7_3	marshall8_3 marshall1_4	marshall2_4	marshall3_4	marshall4_4	marshall5_4	marshall6_4	marshall7_4	marshall8_4 marshall1_5	marshall2_5	marshall3_5	marshall4_5	marshall5_5	marshall6_5	marshall7_5	marshall8_5 marshall1_6	marshall2_6	marshall3_6	marshall4_6	marshall5_6	marshall6_6	marshall7_6	marshall8_6 marshall1_7	marshall2_7	marshall3_7	marshall4_7	marshall5_7	marshall6_7	marshall7_7	marshall8_7 marshall1_8	marshall2_8	marshall3_8	marshall4_8	marshall5_8	marshall6_8	marshall7_8	marshall8_8 symmetry1_1	symmetry2_1	symmetry3_1	symmetry4_1	symmetry5_1	symmetry6_1	symmetry7_1	symmetry8_1 symmetry1_2	symmetry2_2	symmetry3_2	symmetry4_2	symmetry5_2	symmetry6_2	symmetry7_2	symmetry8_2 symmetry1_3	symmetry2_3	symmetry3_3	symmetry4_3	symmetry5_3	symmetry6_3	symmetry7_3	symmetry8_3 symmetry1_4	symmetry2_4	symmetry3_4	symmetry4_4	symmetry5_4	symmetry6_4	symmetry7_4	symmetry8_4 symmetry1_5	symmetry2_5	symmetry3_5	symmetry4_5	symmetry5_5	symmetry6_5	symmetry7_5	symmetry8_5 symmetry1_6	symmetry2_6	symmetry3_6	symmetry4_6	symmetry5_6	symmetry6_6	symmetry7_6	symmetry8_6 symmetry1_7	symmetry2_7	symmetry3_7	symmetry4_7	symmetry5_7	symmetry6_7	symmetry7_7	symmetry8_7 symmetry1_8	symmetry2_8	symmetry3_8	symmetry4_8	symmetry5_8	symmetry6_8	symmetry7_8	symmetry8_8"
global results1 "b1_1	b1_2	b1_3	b1_4	b1_5	b1_6	b1_7	b1_8	c1	d1	w1_age	w1_osob	w1_child	w1_empstat	w1_educlow	w1_educmid	w1_v1	w1_v2	w1_v3	w1_v4	w1_v5	w1_v6	w1_praha	w1_city_size	w1_rok_year	a1	b2_1	b2_2	b2_3	b2_4	b2_5	b2_6	b2_7	b2_8	c2	d2	w2_age	w2_osob	w2_child	w2_empstat	w2_educlow	w2_educmid	w2_v1	w2_v2	w2_v3	w2_v4	w2_v5	w2_v6	w2_praha	w2_city_size	w2_rok_year	a2	b3_1	b3_2	b3_3	b3_4	b3_5	b3_6	b3_7	b3_8	c3	d3	w3_age	w3_osob	w3_child	w3_empstat	w3_educlow	w3_educmid	w3_v1	w3_v2	w3_v3	w3_v4	w3_v5	w3_v6	w3_praha	w3_city_size	w3_rok_year	a3	b4_1	b4_2	b4_3	b4_4	b4_5	b4_6	b4_7	b4_8	c4	d4	w4_age	w4_osob	w4_child	w4_empstat	w4_educlow	w4_educmid	w4_v1	w4_v2	w4_v3	w4_v4	w4_v5	w4_v6	w4_praha	w4_city_size	w4_rok_year	a4	b5_1	b5_2	b5_3	b5_4	b5_5	b5_6	b5_7	b5_8	c5	d5	w5_age	w5_osob	w5_child	w5_empstat	w5_educlow	w5_educmid	w5_v1	w5_v2	w5_v3	w5_v4	w5_v5	w5_v6	w5_praha	w5_city_size	w5_rok_year	a5	b6_1	b6_2	b6_3	b6_4	b6_5	b6_6	b6_7	b6_8	c6	d6	w6_age	w6_osob	w6_child	w6_empstat	w6_educlow	w6_educmid	w6_v1	w6_v2	w6_v3	w6_v4	w6_v5	w6_v6	w6_praha	w6_city_size	w6_rok_year	a6	b7_1	b7_2	b7_3	b7_4	b7_5	b7_6	b7_7	b7_8	c7	d7	w7_age	w7_osob	w7_child	w7_empstat	w7_educlow	w7_educmid	w7_v1	w7_v2	w7_v3	w7_v4	w7_v5	w7_v6	w7_praha	w7_city_size	w7_rok_year	a7	b8_1	b8_2	b8_3	b8_4	b8_5	b8_6	b8_7	a8	c8	d8	b8_8	w8_age	w8_osob	w8_child	w8_empstat	w8_educlow	w8_educmid	w8_v1	w8_v2	w8_v3	w8_v4	w8_v5	w8_v6	w8_praha	w8_city_size	w8_rok_year"
global results2 "IncElast_1 IncElast_2 IncElast_3 IncElast_4 IncElast_5 IncElast_6 IncElast_7 IncElast_8 hicksian1_1 hicksian2_1 hicksian3_1	hicksian4_1	hicksian5_1	hicksian6_1	hicksian7_1	hicksian8_1 hicksian1_2	hicksian2_2	hicksian3_2	hicksian4_2	hicksian5_2	hicksian6_2	hicksian7_2	hicksian8_2 hicksian1_3	hicksian2_3	hicksian3_3	hicksian4_3	hicksian5_3	hicksian6_3	hicksian7_3	hicksian8_3 hicksian1_4	hicksian2_4	hicksian3_4	hicksian4_4	hicksian5_4	hicksian6_4	hicksian7_4	hicksian8_4 hicksian1_5	hicksian2_5	hicksian3_5	hicksian4_5	hicksian5_5	hicksian6_5	hicksian7_5	hicksian8_5 hicksian1_6	hicksian2_6	hicksian3_6	hicksian4_6	hicksian5_6	hicksian6_6	hicksian7_6	hicksian8_6 hicksian1_7	hicksian2_7	hicksian3_7	hicksian4_7	hicksian5_7	hicksian6_7	hicksian7_7	hicksian8_7 hicksian1_8	hicksian2_8	hicksian3_8	hicksian4_8	hicksian5_8	hicksian6_8	hicksian7_8	hicksian8_8 marshall1_1	marshall2_1	marshall3_1	marshall4_1	marshall5_1	marshall6_1	marshall7_1	marshall8_1 marshall1_2	marshall2_2	marshall3_2	marshall4_2	marshall5_2	marshall6_2	marshall7_2	marshall8_2 marshall1_3	marshall2_3	marshall3_3	marshall4_3	marshall5_3	marshall6_3	marshall7_3	marshall8_3 marshall1_4	marshall2_4	marshall3_4	marshall4_4	marshall5_4	marshall6_4	marshall7_4	marshall8_4 marshall1_5	marshall2_5	marshall3_5	marshall4_5	marshall5_5	marshall6_5	marshall7_5	marshall8_5 marshall1_6	marshall2_6	marshall3_6	marshall4_6	marshall5_6	marshall6_6	marshall7_6	marshall8_6 marshall1_7	marshall2_7	marshall3_7	marshall4_7	marshall5_7	marshall6_7	marshall7_7	marshall8_7 marshall1_8	marshall2_8	marshall3_8	marshall4_8	marshall5_8	marshall6_8	marshall7_8	marshall8_8 symmetry1_1	symmetry2_1	symmetry3_1	symmetry4_1	symmetry5_1	symmetry6_1	symmetry7_1	symmetry8_1 symmetry1_2	symmetry2_2	symmetry3_2	symmetry4_2	symmetry5_2	symmetry6_2	symmetry7_2	symmetry8_2 symmetry1_3	symmetry2_3	symmetry3_3	symmetry4_3	symmetry5_3	symmetry6_3	symmetry7_3	symmetry8_3 symmetry1_4	symmetry2_4	symmetry3_4	symmetry4_4	symmetry5_4	symmetry6_4	symmetry7_4	symmetry8_4 symmetry1_5	symmetry2_5	symmetry3_5	symmetry4_5	symmetry5_5	symmetry6_5	symmetry7_5	symmetry8_5 symmetry1_6	symmetry2_6	symmetry3_6	symmetry4_6	symmetry5_6	symmetry6_6	symmetry7_6	symmetry8_6 symmetry1_7	symmetry2_7	symmetry3_7	symmetry4_7	symmetry5_7	symmetry6_7	symmetry7_7	symmetry8_7 symmetry1_8	symmetry2_8	symmetry3_8	symmetry4_8	symmetry5_8	symmetry6_8	symmetry7_8	symmetry8_8"

clear all
pause off
set mem 1G
set more off

***********************************
****** STONE PRICES 		 ******
***********************************
capture program drop stone_price_estimation
program define stone_price_estimation

use "$data/Merge/sru_all_$numdata.dta"
keep if year >= $startyear &  year <= $endyear 

* another option to include in demographics - type of the city - City-size - vel (1-9) or typ (1-3) *tabulate typ, gen(typ)

foreach n in $group { // type of categorization into groups
gen exp = exptotal`n' // total expenditure on group 1 to 8
gen pgn = pgn`n' // weighted average on prices on all exps on group 1 to 8

foreach g in 1 2 3 4 5 6 7 8 { // numbers of the groups
gen w`g' = w`g'_n`n'
* Where there is a missing data on the expenditure share, impute zero 0
replace w`g' = 0 if w`g' == .
* What prices to use? CPI on household or aggregate level?
gen lnpa`g'_n`n' = ln(pa`g'_n`n') // aggreagate level price indicies
gen lnp`g'_n`n' = ln(p`g'_n`n') // household level price indicies
gen lnp`g' = lnpa`g'_n`n' // household or aggreagate level price indicies (a - lnpa`g'_n`n' )
gen p`g' = pa`g'_n`n' // household or aggreagate level price indicies a
}
}

gen lnexp = ln(exp)
gen moninc = cp_prij

* excise duties $excise_duties - the following variables to be included: exp3900 exp3901 exp3902 exp3903 exp3640 exp2830 exp2840 exp2841 exp2842 exp2850 exp2930 exp2931 exp2932 exp2940 exp2941 exp2942 exp2950 exp2951 exp2952 amount2830 amount2840 amount2841 amount2842 amount2850 amount2930 amount2931 amount2932 amount2940 amount2941 amount2942 amount2950 amount2951 amount2952
global excise_duties "sj exp3900 exp3901 exp3902 exp3903 exp3640 exp2830 exp2840 exp2841 exp2842 exp2850 exp2930 exp2931 exp2932 exp2940 exp2941 exp2942 exp2950 exp2951 exp2952 amount2830 amount2840 amount2841 amount2842 amount2850 amount2930 amount2931 amount2932 amount2940 amount2941 amount2942 amount2950 amount2951 amount2952"
global VAT_selected_groups "amount2010-exp5350" // all expenditure and amount variables in the sru_all_11 file

** Choose the variables
keep zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year kraj skup osob2 age2 sex educ pmes child2 pkoef $excise_duties $VAT_selected_groups
order zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year kraj skup osob2 age2 sex educ pmes child2 pkoef $excise_duties $VAT_selected_groups
 
** Get rid of ouliers - exclude all observations within each commodity group with prices below and above the 1st and the 99th percentiles (TBD)
/*
foreach g in 1 2 3 4 5 6 7 8 {
su lnp`g', det
drop if lnp`g' < r(p5)
drop if lnp`g' > r(p95) 
}
su lnexp, det
drop if lnexp < r(p5)
drop if lnexp > r(p95) 
*/

/* ap and bp are unknown and so we approximate them, ap as w*lnp (Stone price index) and bp as 1 */
gen bp = 1	// the values of a(p) and b(p) are unknown to us and therefore we approximate ln?a(p) using the so called Stone price index (Stone 1954)
gen lnap = 0
local x = $numgoods 
foreach X of numlist 1(1)`x' {
	replace lnap = lnap + w`X'*lnp`X'
}

/* Preparing variables for SURE and IV for total expenditure using monetary income */
gen lnm = lnexp - lnap // defining ln(x/ap) - SURE
gen lnm2 = lnm*lnm/bp // defining 1/bp*ln(x/ap)^2 - SURE
gen lnexp2 = lnexp*lnexp
replace moninc = (moninc) + 1 // + 1 because of the log so that min is zero
gen lnmoninc = ln(moninc)
gen lnmoninc2 = lnmoninc*lnmoninc

/* Insert control function equations for IV for total expenditure using monetary income */
reg lnexp lnp1-lnp8 lnmoninc lnmoninc2 $odems 
predict v1, resid
gen v2=v1*v1
gen v3 = v1*v1*v1

reg lnexp2 lnp1-lnp8 lnmoninc lnmoninc2 $odems
predict v4, resid
gen v5=v4*v4
gen v6 = v4*v4*v4

	** SET UP RESTRICTIONS FOR SYMMETRY ** 
	constraint 1 [w1]lnp2   = [w2]lnp1
	constraint 2 [w1]lnp3   = [w3]lnp1
	constraint 3 [w1]lnp4   = [w4]lnp1
	constraint 4 [w1]lnp5   = [w5]lnp1
	constraint 5 [w1]lnp6   = [w6]lnp1
	constraint 6 [w1]lnp7   = [w7]lnp1

	constraint 7 [w2]lnp3  = [w3]lnp2
	constraint 8 [w2]lnp4  = [w4]lnp2
	constraint 9 [w2]lnp5  = [w5]lnp2
	constraint 10 [w2]lnp6  = [w6]lnp2
	constraint 11 [w2]lnp7  = [w7]lnp2

	constraint 12 [w3]lnp4  = [w4]lnp3
	constraint 13 [w3]lnp5  = [w5]lnp3
	constraint 14 [w3]lnp6  = [w6]lnp3
	constraint 15 [w3]lnp7  = [w7]lnp3

	constraint 16 [w4]lnp5  = [w5]lnp4
	constraint 17 [w4]lnp6  = [w6]lnp4
	constraint 18 [w4]lnp7  = [w7]lnp4

	constraint 19 [w5]lnp6  = [w6]lnp5
	constraint 20 [w5]lnp7  = [w7]lnp5

	constraint 21 [w6]lnp7  = [w7]lnp6

** SET UP HOMOGENEITY RESTRICTIONS ** 
	constraint 22 [w1]lnp8 = - [w1]lnp1 - [w1]lnp2 - [w1]lnp3 - [w1]lnp4 - [w1]lnp5 - [w1]lnp6 - [w1]lnp7
	constraint 23 [w2]lnp8 = - [w2]lnp1 - [w2]lnp2 - [w2]lnp3 - [w2]lnp4 - [w2]lnp5 - [w2]lnp6 - [w2]lnp7 
	constraint 24 [w3]lnp8 = - [w3]lnp1 - [w3]lnp2 - [w3]lnp3 - [w3]lnp4 - [w3]lnp5 - [w3]lnp6 - [w3]lnp7 
	constraint 25 [w4]lnp8 = - [w4]lnp1 - [w4]lnp2 - [w4]lnp3 - [w4]lnp4 - [w4]lnp5 - [w4]lnp6 - [w4]lnp7
	constraint 26 [w5]lnp8 = - [w5]lnp1 - [w5]lnp2 - [w5]lnp3 - [w5]lnp4 - [w5]lnp5 - [w5]lnp6 - [w5]lnp7
	constraint 27 [w6]lnp8 = - [w6]lnp1 - [w6]lnp2 - [w6]lnp3 - [w6]lnp4 - [w6]lnp5 - [w6]lnp6 - [w6]lnp7
	constraint 28 [w7]lnp8 = - [w7]lnp1 - [w7]lnp2 - [w7]lnp3 - [w7]lnp4 - [w7]lnp5 - [w7]lnp6 - [w7]lnp7

** RUN THE DEMAND SYSTEM ** (equation for group 8 missing to satisfy the adding up restrictions)
sureg (w1 lnp1-lnp8 lnm lnm2 $dems) (w2 lnp1-lnp8 lnm lnm2 $dems) (w3 lnp1-lnp8 lnm lnm2 $dems) (w4 lnp1-lnp8 lnm lnm2 $dems) (w5 lnp1-lnp8 lnm lnm2 $dems) (w6 lnp1-lnp8 lnm lnm2 $dems) (w7 lnp1-lnp8 lnm lnm2 $dems), constraints(1-28)
estimates save quaids_stone, replace // http://stata.com/help.cgi?sureg

** SET UP A STORAGE VARIABLE **
foreach X of numlist 1(1)8 {
	gen oa`X' = 0
}

** NOW CAPTURE THE CENTRAL ESTIMATES OF COEFFICIENTS  **
	foreach i of varlist w1-w7 {       
      	foreach j of varlist lnp1-lnp8 lnm lnm2 $dems {
                qui gen `i'_`j' = [`i']_b[`j'] // Saving the parameter coefficients from the regression results SUR
            }       /* end j */          
                qui gen `i'_cons = [`i']_b[_cons] // Saving the parameter coefficients from the regression results SUR
        }   /* end i */   

	foreach i of numlist 1(1)7 {
		rename w`i'_cons a`i' // The estimates of alfa i
		rename w`i'_lnm  c`i' // The estimates of beta
		rename w`i'_lnm2 d`i' // The estimates of gama
		foreach j of numlist 1(1)8 {
			rename w`i'_lnp`j' b`i'_`j' // The estimates of lambda
		}
		gen b8_`i' = b`i'_8 // The remaining estimates of lambda (because of the symmetry assumption)
	}

	gen a8 = 1 - a7 - a6 - a5 - a4 - a3 - a2 - a1 // The remaining estimates of alfa i (because of the adding-up assumption)
	gen c8 = 0 - c7 - c6 - c5 - c4 - c3 - c2 - c1 // The remaining estimates of beta (because of the adding-up assumption)
	gen d8 = 0 - d7 - d6 - d5 - d4 - d3 - d2 - d1 // The remaining estimates of gama (because of the adding-up assumption)
	gen b8_8 = 0 - b7_8 - b6_8 - b5_8 - b4_8 - b3_8 - b2_8 - b1_8  // The remaining estimates of lambda (because of the adding-up assumption)

foreach i of varlist $dems {
		gen w8_`i' = - w7_`i'- w6_`i'- w5_`i'- w4_`i'- w3_`i'- w2_`i'- w1_`i' // The remaining estimates of w - (most likely) due to homogeneity restrictions.
	}

* Computation of lnap (according to the definition, in a few steps)
gen alpha0 = 8.82 // Estimate of alfa 0 - get it by: sum lnexp, det (and input the value that is just below the lowest) TBD IMPUT THE VALUE
replace lnap = alpha0
foreach X of numlist 1(1)8 {
	foreach Y in age osob child empstat educlow educmid v1 v2 v3 v4 v5 v6 praha city_size rok_year {
		replace a`X' = a`X' + w`X'_`Y'*`Y'
	}
}
foreach X of numlist 1(1)8 {
	replace lnap = lnap + (a`X'*lnp`X')
}
foreach X of numlist 1(1)8 {
	foreach Y of numlist 1(1)8 {
		replace lnap = lnap + (0.5*b`X'_`Y'*lnp`X'*lnp`Y')
	}
}

* Computation of bp (according to the definition)
replace bp = (p1^([w1]_b[lnm]))*(p2^([w2]_b[lnm]))*(p3^([w3]_b[lnm]))*(p4^([w4]_b[lnm]))*(p5^([w5]_b[lnm]))*(p6^([w6]_b[lnm]))*(p7^([w7]_b[lnm]))*(p8^(-[w7]_b[lnm]-[w6]_b[lnm]-[w5]_b[lnm]-[w4]_b[lnm]-[w3]_b[lnm]-[w2]_b[lnm]-[w1]_b[lnm]))

* Income deciles variables for descriptive statistics
* pjmov kvantily - better to use xtile (generates values 1 to 10 according to deciles) and pctile
gen prijmy = moninc
_pctile prijmy, n(5) 
return list
gen prij_kvantil = 1 if prijmy <= r(r1)
replace prij_kvantil = 2 if prijmy > r(r1) & prijmy <= r(r2)
replace prij_kvantil = 3 if prijmy > r(r2) & prijmy <= r(r3)
replace prij_kvantil = 4 if prijmy > r(r3) & prijmy <= r(r4)
replace prij_kvantil = 5 if prijmy > r(r4)

* pjmov decily
_pctile prijmy, n(10) 
return list
gen prij_decil = 1 if prijmy <= r(r1)
replace prij_decil = 2 if prijmy > r(r1) & prijmy <= r(r2)
replace prij_decil = 3 if prijmy > r(r2) & prijmy <= r(r3)
replace prij_decil = 4 if prijmy > r(r3) & prijmy <= r(r4)
replace prij_decil = 5 if prijmy > r(r4) & prijmy <= r(r5)
replace prij_decil = 6 if prijmy > r(r5) & prijmy <= r(r6)
replace prij_decil = 7 if prijmy > r(r6) & prijmy <= r(r7)
replace prij_decil = 8 if prijmy > r(r7) & prijmy <= r(r8)
replace prij_decil = 9 if prijmy > r(r8) & prijmy <= r(r9)
replace prij_decil = 10 if prijmy > r(r9)

* pjmov kvantily dle OECD - sj
gen prijmy_oecd = moninc/osob
_pctile prijmy_oecd, n(5) 
return list
gen prij_kvantil_oecd = 1 if prijmy_oecd <= r(r1)
replace prij_kvantil_oecd = 2 if prijmy_oecd > r(r1) & prijmy_oecd <= r(r2)
replace prij_kvantil_oecd = 3 if prijmy_oecd > r(r2) & prijmy_oecd <= r(r3)
replace prij_kvantil_oecd = 4 if prijmy_oecd > r(r3) & prijmy_oecd <= r(r4)
replace prij_kvantil_oecd = 5 if prijmy_oecd > r(r4)

* pjmov decily dle OECD - sj
_pctile prijmy_oecd, n(10) 
return list
gen prij_decil_oecd = 1 if prijmy_oecd <= r(r1)
replace prij_decil_oecd = 2 if prijmy_oecd > r(r1) & prijmy_oecd <= r(r2)
replace prij_decil_oecd = 3 if prijmy_oecd > r(r2) & prijmy_oecd <= r(r3)
replace prij_decil_oecd = 4 if prijmy_oecd > r(r3) & prijmy_oecd <= r(r4)
replace prij_decil_oecd = 5 if prijmy_oecd > r(r4) & prijmy_oecd <= r(r5)
replace prij_decil_oecd = 6 if prijmy_oecd > r(r5) & prijmy_oecd <= r(r6)
replace prij_decil_oecd = 7 if prijmy_oecd > r(r6) & prijmy_oecd <= r(r7)
replace prij_decil_oecd = 8 if prijmy_oecd > r(r7) & prijmy_oecd <= r(r8)
replace prij_decil_oecd = 9 if prijmy_oecd > r(r8) & prijmy_oecd <= r(r9)
replace prij_decil_oecd = 10 if prijmy_oecd > r(r9)




save "$outputs/initial_estimates4.dta", replace
end

***********************************
****** ITERATIVE ESTIMATIONS ******
***********************************
capture program drop iterative_estimation
program define iterative_estimation

use "$outputs/initial_estimates4.dta", clear
cap log close
set logtype text
log using $outputs/iteration_log2.txt, replace

* Re-estimation is iterated 8 times
gen num = 0
while num <= 8 {

cap drop w8_child-w8_sex 
cap drop w8_v1-w8_v6 
cap drop w8_praha 
cap drop w8_city_size 
cap drop w8_rok_year 
cap drop w8_typ1-typ3
cap drop w8_empstat 
cap drop w8_educlow 
cap drop w8_educmid
*cap drop w8_age
cap drop b1_1-b8_8

** SET UP RESTRICTIONS FOR SYMMETRY ** 
qui {
	constraint 1 [w1]lnp2   = [w2]lnp1
	constraint 2 [w1]lnp3   = [w3]lnp1
	constraint 3 [w1]lnp4   = [w4]lnp1
	constraint 4 [w1]lnp5   = [w5]lnp1
	constraint 5 [w1]lnp6   = [w6]lnp1
	constraint 6 [w1]lnp7   = [w7]lnp1

	constraint 7 [w2]lnp3  = [w3]lnp2
	constraint 8 [w2]lnp4  = [w4]lnp2
	constraint 9 [w2]lnp5  = [w5]lnp2
	constraint 10 [w2]lnp6  = [w6]lnp2
	constraint 11 [w2]lnp7  = [w7]lnp2

	constraint 12 [w3]lnp4  = [w4]lnp3
	constraint 13 [w3]lnp5  = [w5]lnp3
	constraint 14 [w3]lnp6  = [w6]lnp3
	constraint 15 [w3]lnp7  = [w7]lnp3

	constraint 16 [w4]lnp5  = [w5]lnp4
	constraint 17 [w4]lnp6  = [w6]lnp4
	constraint 18 [w4]lnp7  = [w7]lnp4

	constraint 19 [w5]lnp6  = [w6]lnp5
	constraint 20 [w5]lnp7  = [w7]lnp5

	constraint 21 [w6]lnp7  = [w7]lnp6

** SET UP HOMOGENEITY RESTRICTIONS ** 
	constraint 22 [w1]lnp8 = - [w1]lnp1 - [w1]lnp2 - [w1]lnp3 - [w1]lnp4 - [w1]lnp5 - [w1]lnp6 - [w1]lnp7
	constraint 23 [w2]lnp8 = - [w2]lnp1 - [w2]lnp2 - [w2]lnp3 - [w2]lnp4 - [w2]lnp5 - [w2]lnp6 - [w2]lnp7 
	constraint 24 [w3]lnp8 = - [w3]lnp1 - [w3]lnp2 - [w3]lnp3 - [w3]lnp4 - [w3]lnp5 - [w3]lnp6 - [w3]lnp7 
	constraint 25 [w4]lnp8 = - [w4]lnp1 - [w4]lnp2 - [w4]lnp3 - [w4]lnp4 - [w4]lnp5 - [w4]lnp6 - [w4]lnp7
	constraint 26 [w5]lnp8 = - [w5]lnp1 - [w5]lnp2 - [w5]lnp3 - [w5]lnp4 - [w5]lnp5 - [w5]lnp6 - [w5]lnp7
	constraint 27 [w6]lnp8 = - [w6]lnp1 - [w6]lnp2 - [w6]lnp3 - [w6]lnp4 - [w6]lnp5 - [w6]lnp6 - [w6]lnp7
	constraint 28 [w7]lnp8 = - [w7]lnp1 - [w7]lnp2 - [w7]lnp3 - [w7]lnp4 - [w7]lnp5 - [w7]lnp6 - [w7]lnp7
} /* end quietly */

replace lnm = lnexp - lnap  // re-defining ln(x/ap) using the new lnap for SUR
replace lnm2 = lnm*lnm/bp // re-defining 1/bp*ln(x/ap)^2 using hte new bp for SUR

** RUN THE DEMAND SYSTEM ** 
sureg (w1 lnp1-lnp8 lnm lnm2 $dems) (w2 lnp1-lnp8 lnm lnm2 $dems) (w3 lnp1-lnp8 lnm lnm2 $dems) (w4 lnp1-lnp8 lnm lnm2 $dems) (w5 lnp1-lnp8 lnm lnm2 $dems) (w6 lnp1-lnp8 lnm lnm2 $dems) (w7 lnp1-lnp8 lnm lnm2 $dems), constraints(1-28)
estimates save quaids_iter, replace

cap drop w8_*

** NOW CAPTURE THE CENTRAL ESTIMATES OF COEFFICIENTS  **
	foreach i of varlist w1-w7 {           
      	foreach j of varlist lnp1-lnp8 lnm lnm2 $dems  {
                qui gen `i'_`j' = [`i']_b[`j'] // Saving the parameter coefficients from the regression results SUR
            }   /* end j */
                qui gen `i'_cons = [`i']_b[_cons] // Saving the parameter coefficients from the regression results SUR
        }   /* end i */   
	
	foreach i of numlist 1(1)7 {
		rename w`i'_cons a`i' // The estimates of alfa i
		rename w`i'_lnm  c`i' // The estimates of beta
		rename w`i'_lnm2 d`i' // The estimates of lambda		
		foreach j of numlist 1(1)8 {
			rename w`i'_lnp`j' b`i'_`j' // The estimates of gamma
		}
		gen b8_`i' = b`i'_8 // The remaining estimates of gamma (because of the symmetry assumption)
	}

	gen a8 = 1 - a7 - a6 - a5 - a4 - a3 - a2 - a1 // The remaining estimates of alfa i (because of the adding-up assumption)
	gen c8 = 0 - c7 - c6 - c5 - c4 - c3 - c2 - c1 // The remaining estimates of beta (because of the adding-up assumption)
	gen d8 = 0 - d7 - d6 - d5 - d4 - d3 - d2 - d1 // The remaining estimates of lambda (because of the adding-up assumption)
	gen b8_8 = 0 - b7_8 - b6_8 - b5_8 - b4_8 - b3_8 - b2_8 - b1_8  // The remaining estimates of gamma (because of the adding-up assumption)	
	
foreach i of varlist $dems {
		gen w8_`i' = - w7_`i'- w6_`i'- w5_`i'- w4_`i'- w3_`i'- w2_`i'- w1_`i' // The remaining estimates of w - (most likely) due to homogeneity restrictions.
	}

* Computation of lnap (according to the definition, in a few steps)
foreach X of numlist 1(1)8 {
	replace oa`X' = a`X' // Getting the initial estimates of alfa i from iterative estimation estimation into oa
	foreach Y in age osob child empstat educlow educmid v1 v2 v3 v4 v5 v6 praha city_size rok_year {
		replace a`X' = a`X' + w`X'_`Y'*`Y'
	}
}	
replace lnap = alpha0
foreach X of numlist 1(1)8 {
	replace lnap = lnap + (a`X'*lnp`X')
}
foreach X of numlist 1(1)8 {
	foreach Y of numlist 1(1)8 {
		replace lnap = lnap + (0.5*b`X'_`Y'*lnp`X'*lnp`Y')
	}
}

* Computation of bp (according to the definition)
replace bp = (p1^([w1]_b[lnm]))*(p2^([w2]_b[lnm]))*(p3^([w3]_b[lnm]))*(p4^([w4]_b[lnm]))*(p5^([w5]_b[lnm]))*(p6^([w6]_b[lnm]))*(p7^([w7]_b[lnm]))*(p8^(-[w7]_b[lnm]-[w6]_b[lnm]-[w5]_b[lnm]-[w4]_b[lnm]-[w3]_b[lnm]-[w2]_b[lnm]-[w1]_b[lnm]))

replace num = num + 1

} /* end while */

log close

predict w1_n, equation(#1)
predict w2_n, equation(#2)
predict w3_n, equation(#3)
predict w4_n, equation(#4)
predict w5_n, equation(#5)
predict w6_n, equation(#6)
predict w7_n, equation(#7)

gen w8_n = 1 - w7_n - w6_n - w5_n - w4_n - w3_n - w2_n - w1_n
	
preserve

foreach X of numlist 1(1)8 {
	replace a`X' = oa`X'  // Getting the initial estimates of alfa i from iterative estimation estimation from oa into a
}

keep if _n==1

gen end = 1
drop w*_n
keep end b1_1-b8_8 w*_*

* Saving the coefficients of the QUAIDS model in a file
save coeficients4.dta, replace

// a - The estimates of alfa i
// b - The estimates of gamma
// c - The estimates of beta
// d - The estimates of lambda	

restore

save post_estimates4.dta, replace

*order $results1
*save parameters_elasticities_1.dta, replace // parameters of QUAIDS saved  
end


***********************************
****** DO ELASTICITIES CALCS ******
***********************************
capture program drop elasticities
program define elasticities

use post_estimates4.dta, clear

****** DO ELASTICITIES CALCS ****** - option 1 - for a representative household - not used/presented
	if "$elast" == "1" {
	 // end elast 1

*** These elasticities are calculated for each household individually, and then a weighted average is constructed, with the weights being equal to the household's share of total sample expenditure for the relevant good 
use post_estimates4.dta, clear
set more off

      foreach i of numlist 1(1)8 {
       		qui gen zz`i' = (w`i'_child*1)+(w`i'_osob duch*2)+(w`i'_sex*1)+(w`i'_empstat*1)+(w`i'_educlow*1)    // We will calculate elasticities at these values of these variables
	} /* end i */

** SETS UP SOME REQUIRED PARAMETERS ** 
	foreach i of numlist 1(1)8 {
		foreach j of numlist 1(1)8 { // Kronecker delta (a value of 1 if the variables are equal, and 0 otherwise)
			qui gen mi`i'_`j' = 0
			qui replace mi`i'_`j' = -1 if `i'==`j'
		} /* end j */
	} /* end i */
	
// a - The estimates of alfa i
// b - The estimates of gamma
// c - The estimates of beta
// d - The estimates of lambda		

/* INCOME ELASTICITIES */
foreach i of numlist 1(1)8 {
        qui gen IncElast_`i' = 1+ (c`i'+((2*d`i')/bp)*(lnexp-lnap))/w`i'_n // Income elasticity
        qui gen IncEffec_`i' = (c`i'+((2*d`i')/bp)*(lnexp-lnap)) // 
} /* end i */

/* PRICE ELASTICITIES */
foreach i of numlist 1(1)8 {
        foreach j of numlist 1(1)8 {
              qui gen marshall`i'_`j' = mi`i'_`j'+(b`i'_`j' - IncEffec_`i'*(a`j' + zz`j' +  (b1_`j'*lnp1)+ (b2_`j'*lnp2)+ (b3_`j'*lnp3)+ (b4_`j'*lnp4)+ (b5_`j'*lnp5)+ (b6_`j'*lnp6)+ (b7_`j'*lnp7)+(b8_`j'*lnp8))-(c`j'*d`i'*(lnexp-lnap)*(lnexp-lnap))/bp)/w`i'_n
              qui gen hicksian`i'_`j' = marshall`i'_`j' + w`j'_n*IncElast_`i'
		  qui gen symmetry`i'_`j' = hicksian`i'_`j'*w`i'_n

        } /* end j */
} /* end i */



** Calculating elasticities at mean
disp "starting the code that calculates elasticities at mean"

foreach x of numlist 1(1)6 {
	replace v`x' = 0
}

replace child = 1
replace osob duch = 2
replace sex = 1
replace empstat = 1
replace educlow = 0
replace educmid = 1
replace praha = 1
replace city_size = 1

foreach x of varlist rok_year {
replace `x' = 0
}


foreach x of varlist p1-p8 exp {
	qui egen m_`x' = mean(`x')
	replace `x' = m_`x'
	drop m_`x'
}

foreach x of numlist 1(1)8 {
	qui replace lnp`x' = ln(p`x')
}

* Limiting the dataset to only one observation/row
keep if _n==1

** REDO LNAP, BP **
foreach X of numlist 1(1)8 {
	replace a`X' = oa`X' // Getting the initial estimates of alfa i from iterative estimation estimation from oa into a
	foreach Y in age osob child empstat educlow educmid v1 v2 v3 v4 v5 v6 praha city_size rok_year {
		replace a`X' = a`X' + w`X'_`Y'*`Y'
	}
}	
replace lnap = alpha0
foreach X of numlist 1(1)8 {
	replace lnap = lnap + (a`X'*lnp`X')
}
foreach X of numlist 1(1)8 {
	foreach Y of numlist 1(1)8 {
		replace lnap = lnap + (0.5*b`X'_`Y'*lnp`X'*lnp`Y')
	}
}
replace bp = (p1^([w1]_b[lnm]))*(p2^([w2]_b[lnm]))*(p3^([w3]_b[lnm]))*(p4^([w4]_b[lnm]))*(p5^([w5]_b[lnm]))*(p6^([w6]_b[lnm]))*(p7^([w7]_b[lnm]))*(p8^(-[w7]_b[lnm]-[w6]_b[lnm]-[w5]_b[lnm]-[w4]_b[lnm]-[w3]_b[lnm]-[w2]_b[lnm]-[w1]_b[lnm]))

** REDO LNM LNM2 **
replace lnexp = ln(exp)
replace lnm = lnexp - lnap
replace lnm2 = lnm*lnm/bp

drop w1_n-w8_n

estimates use quaids_iter

predict w1_n, equation(#1)
predict w2_n, equation(#2)
predict w3_n, equation(#3)
predict w4_n, equation(#4)
predict w5_n, equation(#5)
predict w6_n, equation(#6)
predict w7_n, equation(#7)

gen w8_n = 1 - w7_n - w6_n - w5_n - w4_n - w3_n - w2_n - w1_n

drop IncE* marshall* hicksian* symmetry*

/* INCOME ELASTICITIES */
foreach i of numlist 1(1)8 {
        qui gen IncElast_`i' = 1+ (c`i'+((2*d`i')/bp)*(lnexp-lnap))/w`i'_n
        qui gen IncEffec_`i' = (c`i'+((2*d`i')/bp)*(lnexp-lnap))
} /* end i */

/*
** Added on 120523 - because I want a to be alfa i, not alfa i plus w i _ dems * dems
foreach X of numlist 1(1)8 {
	replace a`X' = oa`X' // Getting the initial estimates of alfa i from iterative estimation estimation from oa into a
}	
*/

/* PRICE ELASTICITIES */
foreach i of numlist 1(1)8 {
        foreach j of numlist 1(1)8 {
              qui gen marshall`i'_`j' = mi`i'_`j'+(b`i'_`j' - IncEffec_`i'*(a`j' + zz`j' + (b1_`j'*lnp1)+ (b2_`j'*lnp2)+ (b3_`j'*lnp3)+ (b4_`j'*lnp4)+ (b5_`j'*lnp5)+ (b6_`j'*lnp6)+ (b7_`j'*lnp7)+(b8_`j'*lnp8))-(c`j'*d`i'*(lnexp-lnap)*(lnexp-lnap))/bp)/w`i'_n
              qui gen hicksian`i'_`j' = marshall`i'_`j' +w`j'_n*IncElast_`i'
		  qui gen symmetry`i'_`j' = hicksian`i'_`j'*w`i'_n
        } /* end j */
} /* end i */
} // end elast 1


****** DO ELASTICITIES CALCS ****** - option 2 - for expenditure-weighted households - used/presented
global elast "2" // I had to include it here as well in order for it to work properly (not sure why)
	if "$elast" == "2" {
		* 2: These elasticities are calculated for each household individually, and then a weighted average is constructed, with the weights being equal to the household's share of total sample expenditure for the relevant good 

*** These elasticities are calculated for each household individually, and then a weighted average is constructed, with the weights being equal to the household's share of total sample expenditure for the relevant good 
use post_estimates4.dta, clear
set more off

      foreach i of numlist 1(1)8 {
       		qui gen zz`i' = (w`i'_age*age)+(w`i'_osob*osob)+(w`i'_child*child)+(w`i'_empstat*empstat)+(w`i'_educmid*educmid)+(w`i'_educlow*educlow)       // We will calculate elasticities at these values of these variables
	} /* end i */

** SETS UP SOME REQUIRED PARAMETERS ** 
	foreach i of numlist 1(1)8 {
		foreach j of numlist 1(1)8 { // Kronecker delta (a value of 1 if the variables are equal, and 0 otherwise)
			qui gen mi`i'_`j' = 0
			qui replace mi`i'_`j' = -1 if `i'==`j'
		} /* end j */
	} /* end i */
	
// a - The estimates of alfa i
// b - The estimates of gamma
// c - The estimates of beta
// d - The estimates of lambda		

/* INCOME ELASTICITIES - for each household*/
foreach i of numlist 1(1)8 {
        qui gen IncElast_`i' = 1+ (c`i'+((2*d`i')/bp)*(lnexp-lnap))/w`i'_n // Income elasticity
        qui gen IncEffec_`i' = (c`i'+((2*d`i')/bp)*(lnexp-lnap)) // 
} /* end i */

/* PRICE ELASTICITIES */
foreach i of numlist 1(1)8 {
		qui gen ew`i' = exp * w`i' // the weights being equal to the household's (share of) total sample expenditure for the relevant good
		
        foreach j of numlist 1(1)8 {
		* for each household individually
          qui gen marshall`i'_`j' = mi`i'_`j'+(b`i'_`j' - IncEffec_`i'*(a`j' + zz`j' +  (b1_`j'*lnp1)+ (b2_`j'*lnp2)+ (b3_`j'*lnp3)+ (b4_`j'*lnp4)+ (b5_`j'*lnp5)+ (b6_`j'*lnp6)+ (b7_`j'*lnp7)+(b8_`j'*lnp8))-(c`j'*d`i'*(lnexp-lnap)*(lnexp-lnap))/bp)/w`i'_n
          qui gen hicksian`i'_`j' = marshall`i'_`j' + w`j'_n*IncElast_`i'
		  qui gen symmetry`i'_`j' = hicksian`i'_`j'*w`i'_n

		  gen marshall_3_`i'_`j' = marshall`i'_`j'  
		  gen hicksian_3_`i'_`j' = hicksian`i'_`j'
		  gen symmetry_3_`i'_`j' = symmetry`i'_`j'
		  
		  * for all households - weighted average
		  egen marshall_2_`i'_`j' = wtmean(marshall`i'_`j'), weight(ew`i')  
		  egen hicksian_2_`i'_`j' = wtmean(hicksian`i'_`j'), weight(ew`i')  
		  egen symmetry_2_`i'_`j' = wtmean(symmetry`i'_`j'), weight(ew`i')  

		  replace marshall`i'_`j' = marshall_2_`i'_`j'
		  replace hicksian`i'_`j' = hicksian_2_`i'_`j'
		  replace symmetry`i'_`j' = symmetry_2_`i'_`j'

		  drop marshall_2_`i'_`j' hicksian_2_`i'_`j' symmetry_2_`i'_`j'
  		  *drop marshall_3_`i'_`j' hicksian_3_`i'_`j' symmetry_3_`i'_`j'

        } /* end j */
		 egen IncElast_2_`i' = wtmean(IncElast_`i'), weight(exp) 
		 *replace IncElast_`i' = . if IncElast_`i' < 0
 		 *egen IncElast_3_`i' = wtmean(IncElast_`i'), weight(exp)  // Looking at what happens to the low average energy income elasticity when we do not include the negative ones
		 replace IncElast_`i' = IncElast_2_`i'

} /* end i */
} // end elast 2


*order $results
*
save parameters_elasticities_2.dta, replace // parameters of QUAIDS and results of elasticities saved - used in Demand_3


****** DO ELASTICITIES CALCS ****** - this code presents and saves the calculated elasticities
cap log close
set logtype text 
log using income_elast_mean4.txt, replace

sum IncElast*

log close

keep if _n==1

order $results
keep $results
save parameters_elasticities.dta, replace // parameters of QUAIDS and results of elasticities saved - in one line for the final results

capture drop number
gen number = 1

preserve
keep number symmetry*
reshape long symmetry1_ symmetry2_ symmetry3_ symmetry4_ symmetry5_ symmetry6_ symmetry7_ symmetry8_ , i(number) j(counter)
list
save symmetry_quaids4.dta, replace
restore
preserve

keep number marshall*
reshape long marshall1_ marshall2_ marshall3_ marshall4_ marshall5_ marshall6_ marshall7_ marshall8_, i(number) j(counter)
list
save marshall_quaids4.dta, replace
restore
preserve

keep number hicksian*
reshape long hicksian1_ hicksian2_ hicksian3_ hicksian4_ hicksian5_ hicksian6_ hicksian7_ hicksian8_, i(number) j(counter)
list
save hicksian_quaids4.dta, replace
restore
end



***********************************
****** BOOTSTRAP ESTIMATIONS ******
***********************************

capture program drop bootstrap_standard_errors
program define bootstrap_standard_errors

* I employ David's strategy for bootstrapping, but otherwise I found the best information here: http://www.ats.ucla.edu/stat/stata/faq/ownboot.htm (bsample, simulate)
	/* THIS SECTION REPEATS MUCH OF THE ABOVE ANALYSIS BUT REPEATS IT 100 TIMES TO GET ESTIMATES OF THE BOOTSTRAPPED STANDARD ERRORS */
	/* IT HAS THE FOLLOWING STAGES:
	1) ESTIMATE THE LINEAR APPROXIMATE VERSION
	2) UPDATE THE PRICE INDICES
	3) ITERATE FOR THE FULL SOLUTION
	4) SAVE THE COEFFICIENTS
	5) DO THE ELASTICITIES AT SET/AVERAGE PRICES, INCOMES AND DEMOGRAPHICS 
	*/

use "$data/Merge/sru_all_$numdata.dta", clear
keep if year >= $startyear &  year <= $endyear 
set more off

foreach n in $group { // type of categorization into groups (1-6)
gen exp = exptotal`n' // total of exps on group 1 to 8
gen pgn = pgn`n' // weighted average on prices on all exps on group 1 to 8

foreach g in 1 2 3 4 5 6 7 8 { // numbers of the groups
gen w`g' = w`g'_n`n'
* Where there is a missing data on the expenditure share, impute zero 0
replace w`g' = 0 if w`g' == .
* What prices to use? CPI on household or aggregate level?
gen lnpa`g'_n`n' = ln(pa`g'_n`n') // aggreagate level price indicies
gen lnp`g'_n`n' = ln(p`g'_n`n') // household level price indicies
gen lnp`g' = lnpa`g'_n`n' // household or aggreagate level price indicies (a - lnpa`g'_n`n' )
gen p`g' = pa`g'_n`n' // household or aggreagate level price indicies a
}
}

gen lnexp = ln(exp)
gen moninc = cp_prij

** Choose the variables
keep zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year kraj skup osob2 age2 sex educ pmes child2 pkoef
order zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year kraj skup osob2 age2 sex educ pmes child2 pkoef
 


gen num = 1
gen end = 1

while num <= $numbootstrap { // setting up the number of iterations - should be at least around 500
    replace num = num + 1
	preserve
	bsample

	** SET UP INITIAL STONE PRICES ** 
	* ap and bp are unknown and so we approximate them, ap as w*lnp (Stone price index) and bp as 1 */
gen bp = 1	// the values of a(p) and b(p) are unknown to us and therefore we approximate ln?a(p) using the so called Stone price index (Stone 1954)
gen lnap = 0
local x = $numgoods 
foreach X of numlist 1(1)`x' {
	replace lnap = lnap + w`X' * lnp`X'
}

/* Preparing variables for SURE and IV for total expenditure using monetary income */
gen lnm = lnexp - lnap // defining ln(x/ap) - SURE
gen lnm2 = lnm*lnm/bp // defining 1/bp*ln(x/ap)^2 - SURE
gen lnexp2 = lnexp*lnexp
replace moninc = (moninc) + 1 // + 1 because of the log so that min is zero
gen lnmoninc = ln(moninc)
gen lnmoninc2 = lnmoninc*lnmoninc

/* Insert control function equations for IV for total expenditure using monetary income */
reg lnexp lnp1-lnp8 lnmoninc lnmoninc2 $odems 
predict v1, resid
gen v2=v1*v1
gen v3 = v1*v1*v1

reg lnexp2 lnp1-lnp8 lnmoninc lnmoninc2 $odems
predict v4, resid
gen v5=v4*v4
gen v6 = v4*v4*v4

	** SET UP RESTRICTIONS FOR SYMMETRY ** 
	constraint 1 [w1]lnp2   = [w2]lnp1
	constraint 2 [w1]lnp3   = [w3]lnp1
	constraint 3 [w1]lnp4   = [w4]lnp1
	constraint 4 [w1]lnp5   = [w5]lnp1
	constraint 5 [w1]lnp6   = [w6]lnp1
	constraint 6 [w1]lnp7   = [w7]lnp1

	constraint 7 [w2]lnp3  = [w3]lnp2
	constraint 8 [w2]lnp4  = [w4]lnp2
	constraint 9 [w2]lnp5  = [w5]lnp2
	constraint 10 [w2]lnp6  = [w6]lnp2
	constraint 11 [w2]lnp7  = [w7]lnp2

	constraint 12 [w3]lnp4  = [w4]lnp3
	constraint 13 [w3]lnp5  = [w5]lnp3
	constraint 14 [w3]lnp6  = [w6]lnp3
	constraint 15 [w3]lnp7  = [w7]lnp3

	constraint 16 [w4]lnp5  = [w5]lnp4
	constraint 17 [w4]lnp6  = [w6]lnp4
	constraint 18 [w4]lnp7  = [w7]lnp4

	constraint 19 [w5]lnp6  = [w6]lnp5
	constraint 20 [w5]lnp7  = [w7]lnp5

	constraint 21 [w6]lnp7  = [w7]lnp6

** SET UP HOMOGENEITY RESTRICTIONS ** 
	constraint 22 [w1]lnp8 = - [w1]lnp1 - [w1]lnp2 - [w1]lnp3 - [w1]lnp4 - [w1]lnp5 - [w1]lnp6 - [w1]lnp7
	constraint 23 [w2]lnp8 = - [w2]lnp1 - [w2]lnp2 - [w2]lnp3 - [w2]lnp4 - [w2]lnp5 - [w2]lnp6 - [w2]lnp7 
	constraint 24 [w3]lnp8 = - [w3]lnp1 - [w3]lnp2 - [w3]lnp3 - [w3]lnp4 - [w3]lnp5 - [w3]lnp6 - [w3]lnp7 
	constraint 25 [w4]lnp8 = - [w4]lnp1 - [w4]lnp2 - [w4]lnp3 - [w4]lnp4 - [w4]lnp5 - [w4]lnp6 - [w4]lnp7
	constraint 26 [w5]lnp8 = - [w5]lnp1 - [w5]lnp2 - [w5]lnp3 - [w5]lnp4 - [w5]lnp5 - [w5]lnp6 - [w5]lnp7
	constraint 27 [w6]lnp8 = - [w6]lnp1 - [w6]lnp2 - [w6]lnp3 - [w6]lnp4 - [w6]lnp5 - [w6]lnp6 - [w6]lnp7
	constraint 28 [w7]lnp8 = - [w7]lnp1 - [w7]lnp2 - [w7]lnp3 - [w7]lnp4 - [w7]lnp5 - [w7]lnp6 - [w7]lnp7

** RUN THE DEMAND SYSTEM ** (equation for group 8 missing to satisfy the adding up restrictions)
sureg (w1 lnp1-lnp8 lnm lnm2 $dems) (w2 lnp1-lnp8 lnm lnm2 $dems) (w3 lnp1-lnp8 lnm lnm2 $dems) (w4 lnp1-lnp8 lnm lnm2 $dems) (w5 lnp1-lnp8 lnm lnm2 $dems) (w6 lnp1-lnp8 lnm lnm2 $dems) (w7 lnp1-lnp8 lnm lnm2 $dems), constraints(1-28)
estimates save quaids_stone, replace // http://stata.com/help.cgi?sureg

** SET UP A STORAGE VARIABLE **
foreach X of numlist 1(1)8 {
	gen oa`X' = 0
}

** NOW CAPTURE THE CENTRAL ESTIMATES OF COEFFICIENTS  **
	foreach i of varlist w1-w7 {       
      	foreach j of varlist lnp1-lnp8 lnm lnm2 $dems {
                qui gen `i'_`j' = [`i']_b[`j'] // Saving the parameter coefficients from the regression results SUR
            }       /* end j */          
                qui gen `i'_cons = [`i']_b[_cons] // Saving the parameter coefficients from the regression results SUR
        }   /* end i */   

	foreach i of numlist 1(1)7 {
		rename w`i'_cons a`i' // The estimates of alfa i
		rename w`i'_lnm  c`i' // The estimates of beta
		rename w`i'_lnm2 d`i' // The estimates of gama
		foreach j of numlist 1(1)8 {
			rename w`i'_lnp`j' b`i'_`j' // The estimates of lambda
		}
		gen b8_`i' = b`i'_8 // The remaining estimates of lambda (because of the symmetry assumption)
	}

	gen a8 = 1 - a7 - a6 - a5 - a4 - a3 - a2 - a1 // The remaining estimates of alfa i (because of the adding-up assumption)
	gen c8 = 0 - c7 - c6 - c5 - c4 - c3 - c2 - c1 // The remaining estimates of beta (because of the adding-up assumption)
	gen d8 = 0 - d7 - d6 - d5 - d4 - d3 - d2 - d1 // The remaining estimates of gama (because of the adding-up assumption)
	gen b8_8 = 0 - b7_8 - b6_8 - b5_8 - b4_8 - b3_8 - b2_8 - b1_8  // The remaining estimates of lambda (because of the adding-up assumption)

foreach i of varlist $dems {
		gen w8_`i' = - w7_`i'- w6_`i'- w5_`i'- w4_`i'- w3_`i'- w2_`i'- w1_`i' // The remaining estimates of w - (most likely) due to homogeneity restrictions.
	}

* Computation of lnap (according to the definition, in a few steps)
gen alpha0 = 8.82 // Estimate of alfa 0 - get it by: sum lnexp, det (and input the value that is just below the lowest) TBD IMPUT THE VALUE
replace lnap = alpha0
foreach X of numlist 1(1)8 {
	foreach Y in age osob child empstat educlow educmid v1 v2 v3 v4 v5 v6 praha city_size rok_year {
		replace a`X' = a`X' + w`X'_`Y'*`Y'
	}
}
foreach X of numlist 1(1)8 {
	replace lnap = lnap + (a`X'*lnp`X')
}
foreach X of numlist 1(1)8 {
	foreach Y of numlist 1(1)8 {
		replace lnap = lnap + (0.5*b`X'_`Y'*lnp`X'*lnp`Y')
	}
}

* Computation of bp (according to the definition)
replace bp = (p1^([w1]_b[lnm]))*(p2^([w2]_b[lnm]))*(p3^([w3]_b[lnm]))*(p4^([w4]_b[lnm]))*(p5^([w5]_b[lnm]))*(p6^([w6]_b[lnm]))*(p7^([w7]_b[lnm]))*(p8^(-[w7]_b[lnm]-[w6]_b[lnm]-[w5]_b[lnm]-[w4]_b[lnm]-[w3]_b[lnm]-[w2]_b[lnm]-[w1]_b[lnm]))

* End of Stone price estimation and beginnning of Iterative estimation

	gen numy = 0
	while numy<=8 {

cap drop w8_child-w8_sex 
cap drop w8_v1-w8_v6 
cap drop w8_praha 
cap drop w8_city_size
cap drop w8_rok_year 
cap drop w8_typ1-typ3
cap drop w8_empstat 
cap drop w8_educlow 
cap drop w8_educmid
*cap drop w8_age
cap drop b1_1-b8_8

** SET UP RESTRICTIONS FOR SYMMETRY ** 
qui {
	constraint 1 [w1]lnp2   = [w2]lnp1
	constraint 2 [w1]lnp3   = [w3]lnp1
	constraint 3 [w1]lnp4   = [w4]lnp1
	constraint 4 [w1]lnp5   = [w5]lnp1
	constraint 5 [w1]lnp6   = [w6]lnp1
	constraint 6 [w1]lnp7   = [w7]lnp1

	constraint 7 [w2]lnp3  = [w3]lnp2
	constraint 8 [w2]lnp4  = [w4]lnp2
	constraint 9 [w2]lnp5  = [w5]lnp2
	constraint 10 [w2]lnp6  = [w6]lnp2
	constraint 11 [w2]lnp7  = [w7]lnp2

	constraint 12 [w3]lnp4  = [w4]lnp3
	constraint 13 [w3]lnp5  = [w5]lnp3
	constraint 14 [w3]lnp6  = [w6]lnp3
	constraint 15 [w3]lnp7  = [w7]lnp3

	constraint 16 [w4]lnp5  = [w5]lnp4
	constraint 17 [w4]lnp6  = [w6]lnp4
	constraint 18 [w4]lnp7  = [w7]lnp4

	constraint 19 [w5]lnp6  = [w6]lnp5
	constraint 20 [w5]lnp7  = [w7]lnp5

	constraint 21 [w6]lnp7  = [w7]lnp6

** SET UP HOMOGENEITY RESTRICTIONS ** 
	constraint 22 [w1]lnp8 = - [w1]lnp1 - [w1]lnp2 - [w1]lnp3 - [w1]lnp4 - [w1]lnp5 - [w1]lnp6 - [w1]lnp7
	constraint 23 [w2]lnp8 = - [w2]lnp1 - [w2]lnp2 - [w2]lnp3 - [w2]lnp4 - [w2]lnp5 - [w2]lnp6 - [w2]lnp7 
	constraint 24 [w3]lnp8 = - [w3]lnp1 - [w3]lnp2 - [w3]lnp3 - [w3]lnp4 - [w3]lnp5 - [w3]lnp6 - [w3]lnp7 
	constraint 25 [w4]lnp8 = - [w4]lnp1 - [w4]lnp2 - [w4]lnp3 - [w4]lnp4 - [w4]lnp5 - [w4]lnp6 - [w4]lnp7
	constraint 26 [w5]lnp8 = - [w5]lnp1 - [w5]lnp2 - [w5]lnp3 - [w5]lnp4 - [w5]lnp5 - [w5]lnp6 - [w5]lnp7
	constraint 27 [w6]lnp8 = - [w6]lnp1 - [w6]lnp2 - [w6]lnp3 - [w6]lnp4 - [w6]lnp5 - [w6]lnp6 - [w6]lnp7
	constraint 28 [w7]lnp8 = - [w7]lnp1 - [w7]lnp2 - [w7]lnp3 - [w7]lnp4 - [w7]lnp5 - [w7]lnp6 - [w7]lnp7
} /* end quietly */

replace lnm = lnexp - lnap  // re-defining ln(x/ap) using the new lnap for SUR
replace lnm2 = lnm*lnm/bp // re-defining 1/bp*ln(x/ap)^2 using hte new bp for SUR

** RUN THE DEMAND SYSTEM ** 
sureg (w1 lnp1-lnp8 lnm lnm2 $dems) (w2 lnp1-lnp8 lnm lnm2 $dems) (w3 lnp1-lnp8 lnm lnm2 $dems) (w4 lnp1-lnp8 lnm lnm2 $dems) (w5 lnp1-lnp8 lnm lnm2 $dems) (w6 lnp1-lnp8 lnm lnm2 $dems) (w7 lnp1-lnp8 lnm lnm2 $dems), constraints(1-28)
estimates save quaids_iter, replace

cap drop w8_*

** NOW CAPTURE THE CENTRAL ESTIMATES OF COEFFICIENTS  **
	foreach i of varlist w1-w7 {           
      	foreach j of varlist lnp1-lnp8 lnm lnm2 $dems  {
                qui gen `i'_`j' = [`i']_b[`j'] // Saving the parameter coefficients from the regression results SUR
            }   /* end j */
                qui gen `i'_cons = [`i']_b[_cons] // Saving the parameter coefficients from the regression results SUR
        }   /* end i */   
	
	foreach i of numlist 1(1)7 {
		rename w`i'_cons a`i' // The estimates of alfa i
		rename w`i'_lnm  c`i' // The estimates of beta
		rename w`i'_lnm2 d`i' // The estimates of lambda		
		foreach j of numlist 1(1)8 {
			rename w`i'_lnp`j' b`i'_`j' // The estimates of gamma
		}
		gen b8_`i' = b`i'_8 // The remaining estimates of gamma (because of the symmetry assumption)
	}

	gen a8 = 1 - a7 - a6 - a5 - a4 - a3 - a2 - a1 // The remaining estimates of alfa i (because of the adding-up assumption)
	gen c8 = 0 - c7 - c6 - c5 - c4 - c3 - c2 - c1 // The remaining estimates of beta (because of the adding-up assumption)
	gen d8 = 0 - d7 - d6 - d5 - d4 - d3 - d2 - d1 // The remaining estimates of lambda (because of the adding-up assumption)
	gen b8_8 = 0 - b7_8 - b6_8 - b5_8 - b4_8 - b3_8 - b2_8 - b1_8  // The remaining estimates of gamma (because of the adding-up assumption)	
	
foreach i of varlist $dems {
		gen w8_`i' = - w7_`i'- w6_`i'- w5_`i'- w4_`i'- w3_`i'- w2_`i'- w1_`i' // The remaining estimates of w - (most likely) due to homogeneity restrictions.
	}

* Computation of lnap (according to the definition, in a few steps)
foreach X of numlist 1(1)8 {
	replace oa`X' = a`X' // Getting the initial estimates of alfa i from iterative estimation estimation into oa
	foreach Y in age osob child empstat educlow educmid v1 v2 v3 v4 v5 v6 praha city_size rok_year {
		replace a`X' = a`X' + w`X'_`Y'*`Y'
	}
}	
replace lnap = alpha0
foreach X of numlist 1(1)8 {
	replace lnap = lnap + (a`X'*lnp`X')
}
foreach X of numlist 1(1)8 {
	foreach Y of numlist 1(1)8 {
		replace lnap = lnap + (0.5*b`X'_`Y'*lnp`X'*lnp`Y')
	}
}

* Computation of bp (according to the definition)
replace bp = (p1^([w1]_b[lnm]))*(p2^([w2]_b[lnm]))*(p3^([w3]_b[lnm]))*(p4^([w4]_b[lnm]))*(p5^([w5]_b[lnm]))*(p6^([w6]_b[lnm]))*(p7^([w7]_b[lnm]))*(p8^(-[w7]_b[lnm]-[w6]_b[lnm]-[w5]_b[lnm]-[w4]_b[lnm]-[w3]_b[lnm]-[w2]_b[lnm]-[w1]_b[lnm]))

	replace numy = numy+1
	}

predict w1_n, equation(#1)
predict w2_n, equation(#2)
predict w3_n, equation(#3)
predict w4_n, equation(#4)
predict w5_n, equation(#5)
predict w6_n, equation(#6)
predict w7_n, equation(#7)

gen w8_n = 1 - w7_n - w6_n - w5_n - w4_n - w3_n - w2_n - w1_n
	
		* 2: These elasticities are calculated for each household individually, and then a weighted average is constructed, with the weights being equal to the household's share of total sample expenditure for the relevant good 

*** These elasticities are calculated for each household individually, and then a weighted average is constructed, with the weights being equal to the household's share of total sample expenditure for the relevant good 

      foreach i of numlist 1(1)8 {
       		qui gen zz`i' = (w`i'_age*age)+(w`i'_osob*osob)+(w`i'_child*child)+(w`i'_empstat*empstat)+(w`i'_educmid*educmid)+(w`i'_educlow*educlow)       // We will calculate elasticities at these values of these variables
	} /* end i */

** SETS UP SOME REQUIRED PARAMETERS ** 
	foreach i of numlist 1(1)8 {
		foreach j of numlist 1(1)8 { // Kronecker delta (a value of 1 if the variables are equal, and 0 otherwise)
			qui gen mi`i'_`j' = 0
			qui replace mi`i'_`j' = -1 if `i'==`j'
		} /* end j */
	} /* end i */
	
// a - The estimates of alfa i
// b - The estimates of gamma
// c - The estimates of beta
// d - The estimates of lambda		

/* INCOME ELASTICITIES - for each household*/
foreach i of numlist 1(1)8 {
        qui gen IncElast_`i' = 1+ (c`i'+((2*d`i')/bp)*(lnexp-lnap))/w`i'_n // Income elasticity
        qui gen IncEffec_`i' = (c`i'+((2*d`i')/bp)*(lnexp-lnap)) // 
} /* end i */

/* PRICE ELASTICITIES */
foreach i of numlist 1(1)8 {
		qui gen ew`i' = exp * w`i' // the weights being equal to the household's (share of) total sample expenditure for the relevant good
		
        foreach j of numlist 1(1)8 {
		* for each household individually
          qui gen marshall`i'_`j' = mi`i'_`j'+(b`i'_`j' - IncEffec_`i'*(a`j' + zz`j' +  (b1_`j'*lnp1)+ (b2_`j'*lnp2)+ (b3_`j'*lnp3)+ (b4_`j'*lnp4)+ (b5_`j'*lnp5)+ (b6_`j'*lnp6)+ (b7_`j'*lnp7)+(b8_`j'*lnp8))-(c`j'*d`i'*(lnexp-lnap)*(lnexp-lnap))/bp)/w`i'_n
          qui gen hicksian`i'_`j' = marshall`i'_`j' + w`j'_n*IncElast_`i'
		  qui gen symmetry`i'_`j' = hicksian`i'_`j'*w`i'_n

		  gen marshall_3_`i'_`j' = marshall`i'_`j'  
		  gen hicksian_3_`i'_`j' = hicksian`i'_`j'
		  gen symmetry_3_`i'_`j' = symmetry`i'_`j'
		  
		  * for all households - weighted average
		  egen marshall_2_`i'_`j' = wtmean(marshall`i'_`j'), weight(ew`i')  
		  egen hicksian_2_`i'_`j' = wtmean(hicksian`i'_`j'), weight(ew`i')  
		  egen symmetry_2_`i'_`j' = wtmean(symmetry`i'_`j'), weight(ew`i')  

		  replace marshall`i'_`j' = marshall_2_`i'_`j'
		  replace hicksian`i'_`j' = hicksian_2_`i'_`j'
		  replace symmetry`i'_`j' = symmetry_2_`i'_`j'

		  drop marshall_2_`i'_`j' hicksian_2_`i'_`j' symmetry_2_`i'_`j'
  		  drop marshall_3_`i'_`j' hicksian_3_`i'_`j' symmetry_3_`i'_`j'

        } /* end j */
		 egen IncElast_2_`i' = wtmean(IncElast_`i'), weight(exp) 
		 *replace IncElast_`i' = . if IncElast_`i' < 0
 		 *egen IncElast_3_`i' = wtmean(IncElast_`i'), weight(exp)  // Looking at what happens to the low average energy income elasticity when we do not include the negative ones
		 replace IncElast_`i' = IncElast_2_`i'

} /* end i */
keep if _n==1
		
               * keep b1_1-b8_8 w8_* a? IncElast* marshall* hicksian* symmetry*
				order $results
				keep $results
				append using $outputs/quaids_params.dta               
				save $outputs/quaids_params.dta, replace   

	restore

} /* END WHILE */               
end

***********************************
****** RESULTS PRESENTATION  ******
***********************************
capture program drop results_presentation
program define results_presentation
* Presentation of the results including the standard errors (so it requires the quaids_params results from the bootsrap program)

clear all
use $outputs\parameters_elasticities.dta
set matsize 2000
mkmat $results1, matrix(matrix_results1) 
mkmat $results2, matrix(matrix_results2) 
mkmat $results3, matrix(matrix_results3)

* Results - Elasticities
bstat $results1 using "$outputs\quaids_params.dta", n(33354) stat(matrix_results1)
* Results - QUAIDS parameters and elasticities
bstat $results2 using "$outputs\quaids_params.dta", n(33354) stat(matrix_results2)
* Results - QUAIDS parameters
bstat $results3 using "$outputs\quaids_params.dta", n(33354) stat(matrix_results3)

end


***********************************
****** DESCRIPTIVE STATISTICS *****
***********************************
capture program drop descriptive_statistics
program define descriptive_statistics

clear all
use "$outputs/initial_estimates4.dta"

* Table: Summary statistics
su zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year
bysort year: su zprav year lnp1 lnp2 lnp3 lnp4 lnp5 lnp6 lnp7 lnp8 w1 w2 w3 w4 w5 w6 w7 w8 p1 p2 p3 p4 p5 p6 p7 p8 exp lnexp moninc age osob child empstat educlow educmid praha city_size rok_year

* Table: Summary statistics by Years
tabout year  using "$outputs/tab_year_all.xls", replace format(2) cells(mean p1 mean p2 mean p3 mean p4 mean p5 mean p6 mean p7 mean p8 mean lnp1 mean lnp2 mean lnp3 mean lnp4 mean lnp5 mean lnp6 mean lnp7 mean lnp8 mean w1 mean w2 mean w3 mean w4 mean w5 mean w6 mean w7 mean w8  mean exp mean lnexp mean moninc mean age mean osob mean child mean empstat mean educlow mean educmid mean praha mean city_size) sum

* Graph: Consumtion shares depending on Demographic Characteristics
graph bar w1 w2 w3 w4 w5 w6 w7 w8, over(city_size) stack legend (label(1 "Food") lab(2 "Eating out") lab(3 "Household goods") lab(4 "Clothing") lab(5 "Other services") lab(6 "Transport and recreation") lab(7 "Energy") lab(8 "Other gooods"))
graph bar w1 w2 w3 w4 w5 w6 w7 w8, over() stack legend (label(1 "Food") lab(2 "Eating out") lab(3 "Household goods") lab(4 "Clothing") lab(5 "Other services") lab(6 "Transport and recreation") lab(7 "Energy") lab(8 "Other gooods"))
graph bar w1 w2 w3 w4 w5 w6 w7 w8, over(osob2) stack legend (label(1 "Food") lab(2 "Eating out") lab(3 "Household goods") lab(4 "Clothing") lab(5 "Other services") lab(6 "Transport and recreation") lab(7 "Energy") lab(8 "Other gooods"))
graph bar w1 w2 w3 w4 w5 w6 w7 w8, over(skup) stack legend (label(1 "Food") lab(2 "Eating out") lab(3 "Household goods") lab(4 "Clothing") lab(5 "Other services") lab(6 "Transport and recreation") lab(7 "Energy") lab(8 "Other gooods"))
graph bar w1 w2 w3 w4 w5 w6 w7 w8, over(age2) stack legend (label(1 "Food") lab(2 "Eating out") lab(3 "Household goods") lab(4 "Clothing") lab(5 "Other services") lab(6 "Transport and recreation") lab(7 "Energy") lab(8 "Other gooods"))
graph bar w1 w2 w3 w4 w5 w6 w7 w8, over(educ) stack legend (label(1 "Food") lab(2 "Eating out") lab(3 "Household goods") lab(4 "Clothing") lab(5 "Other services") lab(6 "Transport and recreation") lab(7 "Energy") lab(8 "Other gooods"))

* Graph: Quadratic Regression
foreach g in 1 2 3 4 5 6 7 8 { // numbers of the groups
qui twoway (qfit w`g' lnexp), name(p`g')
}
gr combine p1 p2 p3 p4 p5 p6 p7 p8

twoway (qfit w1 lnexp if sex == 0) (qfit w1 lnexp if sex == 1)


end





exit

***********************************
****** PROGRAMS			     ******
***********************************

stone_price_estimation
iterative_estimation
elasticities
bootstrap_standard_errors
results_presentation
descriptive_statistics
